/**
 * Use the startup phase to tell Thunderbird that it should load
 * our message-content-script.js file whenever a message is displayed
 */

const handleStartup = () => {
  messenger.messageDisplayScripts.register({
    js: [{ file: "/src/scf-content.js" }, { file: "/src/options.js" }],
    css: [{ file: "/src/scf-styles.css" }],
  });
};

console.log("ShowContactFoto addon started.");

const DB_NAME = 'SCFcache';
const DB_VERSION = 2; // Use a long long for this value (don't use a float)
const DB_STORE_NAME = 'fotos';
var db;
var blobURL;
    
function openDb() {
    console.log("openDb background...");
    var req = indexedDB.open(DB_NAME, DB_VERSION);
    req.onsuccess = function (evt) {
      db = this.result;
      console.log("openDb DONE");
    };
    req.onerror = function (evt) {
      console.error("openDb:", evt.target.errorCode);
    };
}

function getObjectStore(store_name, mode) {
    var tx = db.transaction(store_name, mode);
    return tx.objectStore(store_name);
}


var nophotoicon       = browser.runtime.getURL("src/nophoto.png");
var nophotoicon_dark  = browser.runtime.getURL("src/nophoto_dark.png");
var nophotoicon_light = browser.runtime.getURL("src/nophoto_light.png");

function emails_extractor(collected_adr, max_icn) {
	var regexm = /<(.*)>/;
	var emailz = [];
	if (collected_adr.length < max_icn) 
		{var max_limit = collected_adr.length} else {var max_limit= max_icn};
	for (let counter=0; counter<max_limit; counter++) {
			var current = collected_adr[counter];
			var matches = regexm.exec(current);
			if (!matches) {emailz.push(current)}
			else {emailz.push(matches[1])};
	};
	return emailz;
}

function showblob(nameid, picarr) {
    var store = getObjectStore(DB_STORE_NAME, 'readonly');
    var req = store.index('name');
    req.get(nameid).onsuccess = function(evt) {
      if (typeof evt.target.result == 'undefined') {
        picarr.push("none");
        return;
      }
        blobURL = URL.createObjectURL(evt.target.result.blob);
        picarr.push(blobURL);
    };
    req.onerror = function (evt) {
      console.error("deletePublicationFromBib:", evt.target.errorCode);
    };
}


async function source_chooser(emailarr, locsrc, emailfull, skipper, abprio) {
	var picABsrc = [];
	var finalsrc = [];
	var finalemails = [];
	var finalnames = [];
	var reslt1  = await browser.storage.local.get({exist_db:33});
	var reslt3 = reslt1.exist_db;
	if (reslt3 == "yes") {
		openDb();
		for (let r=0; r<emailarr.length; r++) {
			showblob(emailarr[r], locsrc);
			}
		}
	else {
		for (let q=0; q<emailarr.length; q++) {
			locsrc.push("none");
			}
		}
	for (let s=0; s<emailarr.length; s++) {
	const scard = await browser.contacts.quickSearch(emailarr[s]);
		if (scard.length<1) {picABsrc.push("none");} else {
			var photo_file = await browser.contacts.getPhoto(scard[0].id);
			if ( photo_file ) {
				var picURL = URL.createObjectURL(photo_file);
				picABsrc.push(picURL);
			}
			else {picABsrc.push("none");}
		}
		if (locsrc[s] == "none" && picABsrc[s] == "none") {
			if (skipper == false) {
				finalemails.push(emailarr[s]);
				finalnames.push(emailfull[s]);
				finalsrc.push(nophotoicon);
			 }
			}
		else
		{
			if (locsrc[s] == "none" && picABsrc[s] !== "none") {
				finalemails.push(emailarr[s]);
				finalnames.push(emailfull[s]);
				finalsrc.push(picABsrc[s])}
			else
				{
				if (locsrc[s] !== "none" && picABsrc[s] == "none") {
				finalemails.push(emailarr[s]);
				finalnames.push(emailfull[s]);
				finalsrc.push(locsrc[s])}
				else
					{
					if (locsrc[s] !== "none" && picABsrc[s] !== "none" && abprio == true) {
						finalemails.push(emailarr[s]);
						finalnames.push(emailfull[s]);
						finalsrc.push(picABsrc[s])}
					else {
						finalemails.push(emailarr[s]);
						finalnames.push(emailfull[s]);
						finalsrc.push(locsrc[s])}
					}
				}
		}
	};
return {
	mailCollrez		: finalemails,
	mailCollFullrez	: finalnames,
	mailCollsrcRez 	: finalsrc,
	}
}

/**
 * command handler: handles the commands received from the content script
 */
const doHandleCommand = async (message, sender) => {
  const { command } = message;
  const {
    tab: { id: tabId },
  } = sender;

  const messageHeader = await browser.messageDisplay.getDisplayedMessage(tabId);
	var mailCollFull=[];
	var picLocalsrc=[];

// Load Preferences from local storage, or set default if absent
	let SCFprefs = {
		ab_prio     	:	("underfined"? true : priority_place),
		max_icons		: 	("underfined"? 30 : max_icon),
		nophoto			: 	("underfined"? true : nophto),
		nophoto_dark	: 	("underfined"? false : nophto_drk),
		nophoto_light	: 	("underfined"? false : nophto_lht),		
		skip_nofoto	: 	("underfined"? false : lskip_nophoto),
  };
	const fm_store = await browser.storage.local.get(SCFprefs);

	if ( fm_store.nophoto_dark == true ) {nophotoicon = nophotoicon_dark;} else {
		if ( fm_store.nophoto_light == true ) {nophotoicon = nophotoicon_light;}};	

	mailCollFull.push(messageHeader.author);
	mailCollFull.push(...messageHeader.recipients);
	if (messageHeader.ccList.length != 0) {mailCollFull.push(...messageHeader.ccList)};
	if (messageHeader.bccList.length != 0) {mailCollFull.push(...messageHeader.bccList)};

	var mailColl = emails_extractor(mailCollFull, fm_store.max_icons);
    const {mailCollrez, mailCollFullrez, mailCollsrcRez} = await source_chooser(mailColl, picLocalsrc, mailCollFull, fm_store.skip_nofoto, fm_store.ab_prio);

  // check for known commands
  switch (command.toLocaleLowerCase()) {
    case "getnotificationdetails":
      {
        // create the information chunk we want to return to our message content script

        return {
          mailCollb: mailCollrez,
          mailCollFullb: mailCollFullrez,
          localpic: mailCollsrcRez
        };
      }
      break;
      
  }
};

 /* handle the received message by filtering for all messages
 * whose "type" property is set to "command".
 */
const handleMessage = (message, sender, sendResponse) => {
  if (message && message.hasOwnProperty("command")) {
    // if we have a command, return a promise from the command handler
    return doHandleCommand(message, sender);
  }
 };


/**
 * Add a handler for communication with other parts of the extension,
 * like our messageDisplayScript.
*/
 browser.runtime.onMessage.addListener(handleMessage);


/**
 * Execute the startup handler whenever Thunderbird starts
 */
document.addEventListener("DOMContentLoaded", handleStartup);


// COMPOSE Section

messenger.composeScripts.register({
    js: [ { file: "/src/compose.js" } ],
    css: [{ file: "/src/scf-styles.css" }],
  });

async function composeHandle (mesge, snder) {
  const { cmmand } = mesge;
  const { tab: { id: tabLId } } = snder;

  let detls = await browser.compose.getComposeDetails(tabLId);

  var cmailCollFull=[];
  var cpicLocalsrc=[];

// Load Preferences from local storage, or set default if absent
	let SCFprefs = {
		ab_prio     	:	("underfined"? true : priority_place),
		max_icons		: 	("underfined"? 30 : max_icon),
		nophoto			: 	("underfined"? true : nophto),
		nophoto_dark	: 	("underfined"? false : nophto_drk),
		nophoto_light	: 	("underfined"? false : nophto_lht),		
		skip_nofoto		: 	("underfined"? false : cskip_nophoto),
  };
 
	const fm_str = await browser.storage.local.get(SCFprefs);

	if ( fm_str.nophoto_dark == true ) {nophotoicon = nophotoicon_dark;} else {
		if ( fm_str.nophoto_light == true ) {nophotoicon = nophotoicon_light;}};
		
	if (detls.to.length != 0) {cmailCollFull.push(...detls.to)};
	if (detls.cc.length != 0) {cmailCollFull.push(...detls.cc)};
	if (detls.bcc.length != 0) {cmailCollFull.push(...detls.bcc)};
    if (cmailCollFull.length !=0 ) {
		var cmailColl = emails_extractor(cmailCollFull, fm_str.max_icons);
		var cmailRez = await source_chooser(cmailColl, cpicLocalsrc, cmailCollFull, fm_str.skip_nofoto, fm_str.ab_prio);
		var cmailCollrez = cmailRez.mailCollrez;
		var cmailCollFullrez = cmailRez.mailCollFullrez;
		var cmailCollsrcRez = cmailRez.mailCollsrcRez;
    }
    else {
		var cmailCollrez = ["="];
		var cmailCollsrcRez = [nophotoicon];
		cmailCollFullrez.push("NO ADDRESS");
	};
  
  switch(cmmand) {
    case "getAddress":
      {
        // create the information chunk we want to return to our message content script

        return {
          mailCollc: cmailCollrez,
          mailCollFullc: cmailCollFullrez,
          clocalpic: cmailCollsrcRez,
          cmailCollFullc: cmailCollFull
        };
      }
      break;
  };
};  

browser.runtime.onMessage.addListener((mesge, snder) => {
  if (mesge && mesge.hasOwnProperty("cmmand")) {
    return composeHandle(mesge, snder);
  }
}); 

async function remove_grid (tab, details) {
  var compose_body = new DOMParser().parseFromString( details.body, "text/html" );
  let newDetails = details;
  var grid_fotos = compose_body.querySelector("#photo_grid");
  if ( grid_fotos ) {grid_fotos.remove();};
  let body_html = new XMLSerializer().serializeToString( compose_body );
  newDetails.body = body_html;
  return { details: newDetails };
 };

browser.compose.onBeforeSend.addListener(remove_grid);


async function adrsWait (msge, sndr) {
  const { cmnd } = msge;
  const { tab: { id: tId } } = sndr;

  let newdet = await browser.compose.getComposeDetails(tId);
  var newcoll = [];
  	if (newdet.to.length != 0) {newcoll.push(...newdet.to)};
	if (newdet.cc.length != 0) {newcoll.push(...newdet.cc)};
	if (newdet.bcc.length != 0) {newcoll.push(...newdet.bcc)};
      
  switch(cmnd) {
  case "getNewAddr":
      {
        return {
          newrecip : newcoll
        };
      }
      break;
  };
};

browser.runtime.onMessage.addListener((msge, sndr) => {
  if (msge && msge.hasOwnProperty("cmnd")) {
    return adrsWait(msge, sndr);
  }
});
